"use strict";
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var R = require("ramda");
var definitions_interface_1 = require("../definitions.interface");
var definition_1 = require("../definition");
var cond_1 = require("./cond");
var array_1 = require("./array");
var entity_1 = require("./entity");
var type_1 = require("./type");
var nvl_1 = require("./nvl");
var clone_1 = require("./clone");
var wrapper_1 = require("./wrapper");
var filter_1 = require("./filter");
/**
 * @stable [20.12.2019]
 * @param {IGenericBaseFieldEntity} config
 * @returns {AnyT}
 */
exports.buildActualFieldValue = function (config) {
    var emptyValue = config.emptyValue, keepChanges = config.keepChanges, originalValue = config.originalValue, value = config.value;
    var isOriginalValueDefined = type_1.isDef(originalValue);
    var originalOrEmptyValue = isOriginalValueDefined ? originalValue : emptyValue;
    return type_1.isDef(originalOrEmptyValue) && R.equals(value, originalOrEmptyValue)
        ? (isOriginalValueDefined
            ? definition_1.FIELD_VALUE_TO_CLEAR_DIRTY_CHANGES
            : (keepChanges ? value : definition_1.FIELD_VALUE_TO_CLEAR_DIRTY_CHANGES))
        : value;
};
/**
 * @stable [28.10.2019]
 * @param {IGenericBaseFieldEntity} props
 * @returns {boolean}
 */
exports.isFieldInactive = function (props) {
    return wrapper_1.isDisabled(props) || wrapper_1.isReadOnly(props) || wrapper_1.inProgress(props);
};
/**
 * @stable [30.10.2019]
 * @param {AnyT} value
 * @param {AnyT} emptyValue
 * @returns {boolean}
 */
exports.isValuePresent = function (value, emptyValue) {
    return type_1.isDef(value) && !R.equals(value, emptyValue);
};
/**
 * @stable [27.08.2019]
 * @param {EntityIdT} key
 * @returns {string}
 */
exports.dynamicFieldName = function (key) { return "$$dynamicField-" + key; };
/**
 * @stable [21.11.2019]
 * @param {TEntity} object
 * @param {EntityIdT} key
 * @returns {AnyT}
 */
exports.dynamicFieldValue = function (object, key) {
    return Reflect.get(object, exports.dynamicFieldName(key));
};
/**
 * @stable [28.08.2019]
 * @param {TEntity[]} array
 * @param {(itm: TEntity) => EntityIdT} keyAccessor
 * @param {(itm: TEntity) => EntityIdT} valueAccessor
 * @returns {IKeyValue}
 */
exports.fromDynamicFieldsArray = function (array, keyAccessor, valueAccessor) {
    return R.mergeAll(array.map(function (itm) {
        var _a;
        return (_a = {},
            _a[exports.dynamicFieldName(keyAccessor(itm))] = valueAccessor(itm),
            _a);
    }));
};
/**
 * @stable [14.02.2020]
 * @param {EntityIdT[]} array
 * @param {(itm: EntityIdT) => AnyT} valueAccessor
 * @returns {IKeyValue}
 */
exports.fromDynamicFieldsIdsArray = function (array, valueAccessor) {
    return exports.fromDynamicFieldsArray(array, function (itm) { return itm; }, valueAccessor);
};
/**
 * @stable [13.10.2019]
 * @param {MultiFieldEntityT<TEntity extends IEntity>} entity
 * @returns {TEntity[]}
 */
exports.asMultiFieldEntities = function (entity) {
    if (R.isNil(entity)) {
        return definitions_interface_1.UNDEF;
    }
    if (entity_1.isNotMultiEntity(entity)) {
        return entity;
    }
    var multiEntity = entity;
    var sourceEntities = multiEntity.source || [];
    // Fill a cache
    var cachedSourceEntities = new Map();
    sourceEntities.forEach(function (itm) { return cachedSourceEntities.set(itm.id, itm); });
    // Pass a map to optimize
    var editedEntities = exports.asMultiFieldEditedEntities(entity, cachedSourceEntities);
    var cachedEditedEntities = new Map();
    if (array_1.isArrayNotEmpty(editedEntities)) {
        editedEntities.forEach(function (itm) { return cachedEditedEntities.set(itm.id, itm); });
    }
    // Remove the deleted entities
    multiEntity.remove.forEach(function (itm) { return cachedSourceEntities.delete(itm.id); });
    return multiEntity.add.concat(Array.from(cachedSourceEntities.values())
        .map(function (itm) { return nvl_1.nvl(cachedEditedEntities.get(itm.id), itm); }));
};
/**
 * @stable [12.10.2019]
 * @param {MultiFieldEntityT} entity
 * @param {Map<EntityIdT, TItem extends IEntity>} mappedSourcedItems
 * @returns {TItem[]}
 */
exports.asMultiFieldEditedEntities = function (entity, mappedSourcedItems) {
    var multiEntity = entity;
    if (R.isNil(entity)) {
        return definitions_interface_1.UNDEF;
    }
    if (entity_1.isNotMultiEntity(entity)) {
        return [];
    }
    var resultItems = new Map();
    var cachedSourceItems = mappedSourcedItems || new Map();
    if (R.isNil(mappedSourcedItems)) {
        multiEntity.source.forEach(function (sourceEntity) { return cachedSourceItems.set(sourceEntity.id, sourceEntity); });
    }
    multiEntity.edit.forEach(function (editedItem) {
        var editedItemId = editedItem.id;
        var cachedResultItem = resultItems.get(editedItemId);
        // Collect the changes
        var editedItem0 = cachedResultItem || clone_1.shallowClone(cachedSourceItems.get(editedItemId));
        editedItem0[editedItem.name] = editedItem.value;
        if (R.isNil(cachedResultItem)) {
            resultItems.set(editedItemId, editedItem0);
        }
    });
    return Array.from(resultItems.values());
};
/**
 * @stable [22.11.2019]
 * @param {MultiFieldEntityT} entity
 * @returns {TEntity[]}
 */
exports.asMultiFieldAddedEntities = function (entity) {
    if (R.isNil(entity)) {
        return definitions_interface_1.UNDEF;
    }
    if (entity_1.isNotMultiEntity(entity)) {
        return [];
    }
    var multiEntity = entity;
    return multiEntity.add;
};
/**
 * @stable [14.10.2019]
 * @param {NotMultiFieldEntityT} value
 * @returns {IEntity[]}
 */
exports.asEntitiesArray = function (value) {
    return type_1.isPrimitive(value) ? [{ id: value }] : value;
};
/**
 * @stable [14.10.2019]
 * @param {MultiFieldEntityT | EntityIdT} value
 * @returns {number}
 */
exports.asMultiFieldEntitiesLength = function (value) { return cond_1.ifNotNilThanValue(value, function () { return (entity_1.isNotMultiEntity(value)
    ? exports.asEntitiesArray(value)
    : exports.asMultiFieldEntities(value)).length; }, 0); };
/**
 * @stable [14.10.2019]
 * @param {MultiFieldEntityT<TEntity extends IEntity>} value
 * @param {number} entitiesCountLimit
 * @returns {TEntity[]}
 */
exports.asOrderedMultiFieldEntities = function (value, entitiesCountLimit) {
    var result = array_1.makeArray(entitiesCountLimit);
    var multiFieldEntities = exports.asMultiFieldEntities(value);
    if (Array.isArray(multiFieldEntities)) {
        var cursor_1 = 0;
        result.forEach(function (_, index) {
            var entity = multiFieldEntities[index];
            if (!R.isNil(entity)) {
                if (entity.newEntity) {
                    result[entity.index] = entity;
                }
                else {
                    result[cursor_1++] = entity;
                }
            }
        });
    }
    return result;
};
/**
 * @stable [14.10.2019]
 * @param {MultiFieldEntityT<TEntity extends IEntity>} multiFieldEntity
 * @param {(entity: TEntity, index: number) => TResult} mapper
 * @returns {TResult[]}
 */
exports.asMultiFieldMappedEntities = function (multiFieldEntity, mapper) {
    return cond_1.ifNotNilThanValue(exports.asMultiFieldEntities(multiFieldEntity), function (result) { return result.map(mapper); }, definitions_interface_1.UNDEF_SYMBOL);
};
/**
 * @stable [14.10.2019]
 * @param {MultiFieldEntityT<TEntity extends IEntity> | EntityIdT[]} multiFieldEntity
 * @returns {EntityIdT[]}
 */
exports.asMultiFieldMappedEntitiesIds = function (multiFieldEntity) {
    return exports.asMultiFieldMappedEntities(multiFieldEntity, function (entity) { return entity.id; });
};
/**
 * @stable [22.11.2019]
 * @param {string} name
 * @param {AnyT} value
 * @param {TEntity} rawData
 * @param {boolean} newEntity
 * @returns {IMultiItemEntity}
 */
exports.buildMultiItemEntity = function (name, value, rawData, newEntity) {
    return filter_1.defValuesFilter({ id: rawData.id, value: value, name: name, rawData: rawData, newEntity: newEntity });
};
/**
 * @stable [22.11.2019]
 * @param {Partial<IMultiEntity<TEntity extends IEntity>>} initial
 * @returns {IMultiEntity<TEntity extends IEntity>}
 */
exports.multiEntityFactory = function (initial) { return ({
    add: initial.add || [],
    edit: initial.edit || [],
    remove: initial.remove || [],
    source: initial.source || [],
}); };
/**
 * @stable [19.11.2019]
 * @param {TEntity} original
 * @param {MultiFieldEntityT<TEntity extends IEntity>} multiFieldEntity
 * @returns {{id: number}}
 */
exports.buildNewPhantomMultiItem = function (original, multiFieldEntity) { return (__assign({}, original, { id: array_1.arrayNextMinNegativeValue(exports.asMultiFieldMappedEntitiesIds(multiFieldEntity) || []) })); };
//# sourceMappingURL=field.js.map